from __future__ import print_function
import unittest
import IMP.rmf
import IMP.test
import RMF
from IMP.algebra import *


class Tests(IMP.test.TestCase):

    def _show(self, g):
        for i in range(0, g.get_number_of_children()):
            print(i, g.get_child_name(i), g.get_child_is_group(i))

    def test_rt(self):
        """Test round trip"""
        for suffix in [".rmfz", ".rmf3"]:
            m = IMP.Model()
            print("reading pdb")
            name = self.get_tmp_file_name("test_round_trip" + suffix)
            h = IMP.atom.read_pdb(self.get_input_file_name("simple.pdb"), m,
                                  IMP.atom.NonAlternativePDBSelector())
            h.get_is_valid(True)
            IMP.set_log_level(IMP.SILENT)
            IMP.atom.add_bonds(h)
            print("done")
            print("writing hierarchy")
            f = RMF.create_rmf_file(name)
            IMP.rmf.add_hierarchy(f, h)
            IMP.rmf.save_frame(f, str(0))
            print("reopening")
            del f
            print("after closing")
        # print RMF.get_open_hdf5_handle_names()
            f = RMF.open_rmf_file_read_only(name)
            print("reading")
            print("creating")
            h2 = IMP.rmf.create_hierarchies(f, m)
            print("output")
            IMP.atom.show_molecular_hierarchy(h2[0])
            print("loading")
            IMP.rmf.load_frame(f, RMF.FrameID(0))
            print("output")
            IMP.atom.show_molecular_hierarchy(h2[0])
            print("checking")
            self.assertEqual(len(h2), 1)
            self.assertEqual(len(IMP.atom.get_leaves(h)),
                             len(IMP.atom.get_leaves(h2[0])))
            self.assertEqual(
                len([a for a in IMP.atom.get_leaves(h2)
                     if not IMP.atom.Atom.get_is_setup(a)]),
                0)
            self.assertAlmostEqual(IMP.atom.get_mass(IMP.atom.Selection(h)),
                                   IMP.atom.get_mass(IMP.atom.Selection(h2[0])), delta=1e-4)
            if hasattr(IMP.atom, 'get_surface_area'):
                self.assertAlmostEqual(IMP.atom.get_surface_area(h),
                                       IMP.atom.get_surface_area(h2[0]), delta=1e-4)
                self.assertAlmostEqual(IMP.atom.get_volume(h),
                                       IMP.atom.get_volume(h2[0]), delta=1e-4)

    def test_part1(self):
        """Test round trip 1"""
        for suffix in [".rmfz", ".rmf3"]:
            m = IMP.Model()
            print("reading pdb")
            h = IMP.atom.read_pdb(self.get_input_file_name("simple.pdb"), m,
                                  IMP.atom.NonAlternativePDBSelector())
            IMP.set_log_level(IMP.SILENT)
            IMP.atom.add_bonds(h)
            print("done")
            IMP.set_log_level(IMP.VERBOSE)
            print("writing hierarchy")
            name = self.get_tmp_file_name("test_rt_parts" + suffix)
            f = RMF.create_rmf_file(name)
            print(f, type(f))
            IMP.rmf.add_hierarchy(f, h)
            IMP.rmf.save_frame(f, str(0))
            del f
            f = RMF.open_rmf_file_read_only(name)
            print("reading")
            print(f, type(f))
            h2 = IMP.rmf.create_hierarchies(f, m)
            self.assertEqual(len(h2), 1)
            del f

            m = IMP.Model()
            print("reopening")
            f = RMF.open_rmf_file_read_only(name)
            print("reading")
            h2 = IMP.rmf.create_hierarchies(f, m)

    def test_names(self):
        """Test if RMF can recall the molecule name and chain ID"""
        for suffix in [".rmfz", ".rmf3"]:
            m = IMP.Model()
            h = IMP.atom.read_pdb(self.get_input_file_name("simple.pdb"), m,
                                  IMP.atom.NonAlternativePDBSelector())
            chs = IMP.atom.get_by_type(h, IMP.atom.CHAIN_TYPE)
            chs[0].set_name('simple')
            self.assertEqual(chs[0].get_name(), 'simple')

            IMP.set_log_level(IMP.SILENT)
            IMP.atom.add_bonds(h)
            IMP.set_log_level(IMP.VERBOSE)
            name = self.get_tmp_file_name("test_rt_parts" + suffix)
            f = RMF.create_rmf_file(name)
            IMP.rmf.add_hierarchy(f, h)
            IMP.rmf.save_frame(f, str(0))
            del f
            f = RMF.open_rmf_file_read_only(name)
            h2 = IMP.rmf.create_hierarchies(f, m)
            chs2 = IMP.atom.get_by_type(h2[0], IMP.atom.CHAIN_TYPE)
            self.assertEqual(IMP.atom.Chain(chs2[0]).get_id(), 'A')
            self.assertEqual(chs2[0].get_name(), 'simple')

if __name__ == '__main__':
    IMP.test.main()
